﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.ServiceModel;
using System.Text;
using System.Xml;

namespace Microsoft.MSE.Catalog.Contracts.Services
{
    /// <summary>
    /// 
    /// </summary>
    [ServiceContract(Namespace = Schemas.Services, Name = "DesignModel")]
    public interface IDesignModel
    {       

        #region Policies

        [OperationContract]
        PolicyList ListPolicies();

        [OperationContract]
        Guid GetIDForPolicy(string entityName, EntityClass entityClass);

        [OperationContract]
        Guid CreatePolicy(Policy policy);

        [OperationContract]
        void DeletePolicy(Guid ID);

        [OperationContract]
        Policy UpdatePolicy(Policy policy);

        [OperationContract]
        void AssignPolicyAssertionToPolicy(Guid policyID, Guid policyAssertionID, int assertionOrder);

        [OperationContract]
        void RemovePolicyAssertionFromPolicy(Guid policyID, Guid policyAssertionID);

        #endregion

        #region Assertion

        [OperationContract]
        Guid CreatePolicyAssertion(PolicyAssertion policyAssertion);

        [OperationContract]
        void DeletePolicyAssertion(Guid ID);

        [OperationContract]
        PolicyAssertion UpdatePolicyAssertion(PolicyAssertion policyAssertion);

        [OperationContract]
        PolicyAssertion GetPolicyAssertion(Guid ID);

        [OperationContract]
        PolicyAssertionList ListPolicyAssertions();

        [OperationContract]
        PolicyAssertionList ListPolicyAssertionsByPolicy(Guid policyID);

        #endregion

        #region Assertion Type

        [OperationContract]
        Guid CreateAssertionType(AssertionType assertionType);

        [OperationContract]
        void DeleteAssertionType(Guid ID);

        [OperationContract]
        AssertionType GetAssertionType(Guid ID);

        [OperationContract]
        AssertionTypeList ListAssertionType();

        [OperationContract]
        AssertionType UpdateAssertionType(AssertionType assertionType);

        [OperationContract]
        void AssignSubjectToAssertionType(EntityClass entityClass, Guid assertionTypeID);

        [OperationContract]
        void RemoveAllSubjectsFromAssertionType(Guid assertionTypeID);

        [OperationContract]
        EntityClass[] GetAssertionTypeSubjects(Guid assertionTypeID);

        #endregion

        #region Runtime Servers

        [OperationContract]
        RuntimeServerList ListRuntimeServers();

        [OperationContract]
        Guid CreateRuntimeServer(RuntimeServer runtimeServer);

        [OperationContract]
        void DeleteRuntimeServer(Guid ID);

        [OperationContract]
        RuntimeServer UpdateRuntimeServer(RuntimeServer runtimeServer);

        [OperationContract]
        RuntimeServer GetRuntimeServer(Guid ID);

        [OperationContract]
        RuntimeServer GetRuntimeServerWithoutEndpointWSDL(Guid ID);

        #endregion

        #region Schemas

        [OperationContract]
        Guid CreateSchema(Schema schema);

        [OperationContract]
        List<Guid> ImportSchemas(List<Schema> importedSchemas);

        [OperationContract]
        Schema GetSchema(Guid ID);

        [OperationContract]
        SchemaList ListSchemas();

        [OperationContract]
        List<Entity> ListSchemaObjectReferences(Guid ID);

        [OperationContract]
        List<SchemaObject> ListSchemaObjectDependencies(Guid ID);

        [OperationContract]
        Schema UpdateSchema(Schema schema);

        [OperationContract]
        void ValidateSchema(Schema schema);

        [OperationContract]
        void DeleteSchema(Guid ID);

        #endregion

        #region Endpoints

        [OperationContract]
        EndpointList ListEndpoints();

        [OperationContract]
        EndpointList ListEndpointsForRuntimeServer(string runtimeServerName);

        [OperationContract]
        Guid CreateEndpoint(Endpoint endpoint);

        [OperationContract]
        void DeleteEndpoint(Guid ID);

        [OperationContract]
        Endpoint UpdateEndpoint(Endpoint endpoint);

        [OperationContract]
        Endpoint GetEndpointWithoutWSDL(Guid ID);

        [OperationContract]
        Guid GetIDForEndpoint(string name, int port, string scheme);

        [OperationContract]
        Endpoint GetEndpoint(Guid ID, Guid runtimeServerID);

        [OperationContract]
        void AssignAssociatedOperationVerToEndpoint(Guid operationVersionID, Guid endpointID);

        [OperationContract]
        void AssignPublishedOperationVerToEndpoint(Guid operationVersionID, Guid endpointID);

        [OperationContract]
        void RemoveAssociatedOperationVerForEndpoint(Guid operationVersionID, Guid endpointID);

        [OperationContract]
        void RemovePublishedOperationVerForEndpoint(Guid operationVersionID, Guid endpointID);

        #endregion

        #region MSEBindings

        [OperationContract]
        MSEBindingList ListMSEBindings();

        [OperationContract]
        Guid CreateMSEBinding(MSEBinding binding);

        [OperationContract]
        void DeleteMSEBinding(Guid ID);

        [OperationContract]
        MSEBinding UpdateMSEBinding(MSEBinding binding);

        [OperationContract]
        MSEBinding GetMSEBinding(Guid ID);

        [OperationContract]
        void AssignBindingElementsToBinding(Guid bindingID, Guid bindingElementID, int elementOrder);

        [OperationContract]
        void RemoveBindingElementsFromBinding(Guid bindingID, Guid bindingElementID, int elementOrder);

        #endregion

        #region MSEBindingElement

        [OperationContract]
        Guid CreateMSEBindingElement(MSEBindingElement bindingElement);

        [OperationContract]
        MSEBindingElement UpdateMSEBindingElement(MSEBindingElement bindingElement);

        [OperationContract]
        void DeleteMSEBindingElement(Guid ID);

        [OperationContract]
        MSEBindingElement GetMSEBindingElement(Guid ID);

        [OperationContract]
        MSEBindingElementList ListMSEBindingElements();        

        #endregion

        #region MSESystem

        [OperationContract]
        Guid CreateMSESystem(MSESystem mseSystem);

        [OperationContract]
        void DeleteMSESystem(Guid ID);

        [OperationContract]
        MSESystem UpdateMSESystem(MSESystem mseSystem);

        [OperationContract]
        Guid GetIDForMSESystem(string entityName, string uri);

        [OperationContract]
        MSESystem GetMSESystem(Guid ID);

        [OperationContract]
        MSESystemList ListMSESystems();

        #endregion

        #region SystemInstance

        [OperationContract]
        Guid CreateSystemInstance(SystemInstance systemInstance);

        [OperationContract]
        void DeleteSystemInstance(Guid ID);

        [OperationContract]
        SystemInstance UpdateSystemInstance(SystemInstance systemInstance);

        [OperationContract]
        Guid GetIDForSystemInstance(string entityName, string uri, int port);

        [OperationContract]
        SystemInstance GetSystemInstance(Guid ID);

        [OperationContract]
        SystemInstanceList ListSystemInstances();

        #endregion

        #region Resource

        [OperationContract]
        Guid CreateResource(Resource resource);

        [OperationContract]
        void DeleteResource(Guid ID);

        [OperationContract]
        Resource UpdateResource(Resource resource);

        [OperationContract]
        Guid GetIDForResource(string entityName, string scheme, string uri);

        [OperationContract]
        Resource GetResource(Guid ID);

        [OperationContract]
        ResourceList ListResources();

        #endregion

        #region SchemaObject

        [OperationContract]
        SchemaObject GetSchemaObjectByID(Guid schemaObjectID);

        [OperationContract]
        void UpdateSchemaObjectReferences(List<Guid> oldSchemaObjectIDs, Guid newSchemaObjectID, bool removeOldVersion);

        #endregion

        #region DataEntity

        [OperationContract]
        DataEntity GetDataEntity(Guid ID);

        [OperationContract]
        DataEntity CreateDataEntity(DataEntity dataEntity);

        [OperationContract]
        DataEntityList ListDataEntities();

        [OperationContract]
        DataEntity UpdateDataEntity(DataEntity dataEntity);

        [OperationContract]
        void DeleteDataEntity(Guid ID);

        #endregion

        #region Operations

        [OperationContract]
        OperationList ListOperations();

        [OperationContract]
        OperationList ListOperationsBySchema(Guid schemaID);

        [OperationContract]
        OperationList ListOperationsByResource(Guid schemaID);

        [OperationContract]
        Guid CreateOperation(Operation operation);

        [OperationContract]
        void DeleteOperation(Guid ID);

        [OperationContract]
        Operation UpdateOperation(Operation operation);

        [OperationContract]
        Guid GetIDForOperation(string entityName, string fqn);

        #endregion

        #region OperationVersion

        [OperationContract]
        Guid CreateOperationVersion(OperationVersion operationVersion);

        [OperationContract]
        void DeleteOperationVersion(Guid ID);
        
        [OperationContract]
        OperationVersion UpdateOperationVersion(OperationVersion operationVersion);

        [OperationContract]
        OperationVersion GetOperationVersion(Guid ID);

        #endregion

        #region Actors

        [OperationContract]
        ActorList ListActors();

        [OperationContract]
        Guid CreateActor(Actor actor);

        [OperationContract]
        void DeleteActor(Guid ID);

        [OperationContract]
        Actor UpdateActor(Actor actor);

        [OperationContract]
        Actor GetActor(Guid ID);
        #endregion 

        #region Capabilities

        [OperationContract]
        CapabilityList ListCapabilities();

        [OperationContract]
        Guid CreateCapability(Capability capability);

        [OperationContract]
        void DeleteCapability(Guid ID);

        [OperationContract]
        Capability UpdateCapability(Capability capability);

        [OperationContract]
        Capability GetCapability(Guid ID);

        [OperationContract]
        void AssignCapabilityToActor(Guid actorID, Guid capabilityID, Guid sleID);

        [OperationContract]
        void RemoveCapabilityFromActor(Guid actorID, Guid capabilityID);

        [OperationContract]
        void AssignOperationToCapability(Guid capabilityID, Guid operationID);

        [OperationContract]
        void RemoveOperationFromCapability(Guid capabilityID, Guid operationID);

        [OperationContract]
        void AssignTypeToCapability(Guid capabilityID, Guid typeID, Boolean outputFlag);

        [OperationContract]
        void RemoveTypeFromCapability(Guid capabilityID, Guid typeID, Boolean outputFlag);

        #endregion

        #region MSETypes

        [OperationContract]
        MSETypeList ListTypes();

        [OperationContract]
        Guid CreateType(MSEType type);

        [OperationContract]
        void DeleteType(Guid ID);

        [OperationContract]
        MSEType UpdateType(MSEType type);

        [OperationContract]
        MSEType GetType(Guid ID);
        
        [OperationContract]
        void AssignDataEntityToType(Guid typeID, Guid dataEntityID);

        [OperationContract]
        void RemoveDataEntityFromType(Guid typeID, Guid dataEntityID);

        #endregion

        #region SLEs

        [OperationContract]
        SLEList ListSLEs();

        [OperationContract]
        Guid CreateSLE(SLE sle);

        [OperationContract]
        void DeleteSLE(Guid ID);

        [OperationContract]
        SLE UpdateSLE(SLE type);

        [OperationContract]
        SLE GetSLE(Guid ID);

        [OperationContract]
        SLE GetSLEForActorCapability(Guid actorID, Guid capabilityID);       

        #endregion

        //Groups

        #region Actor Groups

        [OperationContract]
        ActorGroup CreateActorGroup(ActorGroup actorGroup);

        [OperationContract]
        ActorGroup UpdateActorGroup(ActorGroup actorGroup);

        [OperationContract]
        ActorGroup GetActorGroup(Guid ID);

        [OperationContract]
        void DeleteActorGroup(Guid ID);

        [OperationContract]
        EntityGroupList ListActorGroups();

        [OperationContract]
        EntityGroupList ListActorGroupsByActor(Guid actorID);

        [OperationContract]
        List<ActorListElement> ListActorsByGroup(Guid ID);

        #endregion

        #region Capability Groups

        [OperationContract]
        CapabilityGroup CreateCapabilityGroup(CapabilityGroup capabilityGroup);

        [OperationContract]
        CapabilityGroup UpdateCapabilityGroup(CapabilityGroup capabilityGroup);

        [OperationContract]
        CapabilityGroup GetCapabilityGroup(Guid ID);

        [OperationContract]
        void DeleteCapabilityGroup(Guid ID);

        [OperationContract]
        EntityGroupList ListCapabilityGroups();

        [OperationContract]
        EntityGroupList ListCapabilityGroupsByCapability(Guid capabilityID);

        [OperationContract]
        List<CapabilityListElement> ListCapabilitiesByGroup(Guid ID);

        #endregion

        #region MSEType Groups

        [OperationContract]
        MSETypeGroup CreateMSETypeGroup(MSETypeGroup mseTypeGroup);

        [OperationContract]
        MSETypeGroup UpdateMSETypeGroup(MSETypeGroup mseTypeGroup);

        [OperationContract]
        MSETypeGroup GetMSETypeGroup(Guid ID);

        [OperationContract]
        void DeleteMSETypeGroup(Guid ID);

        [OperationContract]
        EntityGroupList ListMSETypeGroups();

        [OperationContract]
        EntityGroupList ListMSETypeGroupsByMSEType(Guid mseTypeID);

        [OperationContract]
        List<MSETypeListElement> ListMSETypesByGroup(Guid ID);

        #endregion

        #region SLE Groups

        [OperationContract]
        SLEGroup CreateSLEGroup(SLEGroup sleGroup);

        [OperationContract]
        SLEGroup UpdateSLEGroup(SLEGroup sleGroup);

        [OperationContract]
        SLEGroup GetSLEGroup(Guid ID);

        [OperationContract]
        void DeleteSLEGroup(Guid ID);

        [OperationContract]
        EntityGroupList ListSLEGroups();

        [OperationContract]
        EntityGroupList ListSLEGroupsBySLE(Guid sleID);

        [OperationContract]
        List<SLEListElement> ListSLEsByGroup(Guid ID);

        #endregion

        #region Operation Groups

        [OperationContract]
        OperationGroup CreateOperationGroup(OperationGroup operationGroup);

        [OperationContract]
        OperationGroup UpdateOperationGroup(OperationGroup operationGroup);

        [OperationContract]
        OperationGroup GetOperationGroup(Guid ID);

        [OperationContract]
        void DeleteOperationGroup(Guid ID);

        [OperationContract]
        EntityGroupList ListEntityGroups();

        [OperationContract]
        EntityGroupList ListOperationGroups();

        [OperationContract]
        EntityGroupList ListOperationGroupsByOperation(Guid operationID);

        [OperationContract]
        List<OperationListElement> ListOperationsByGroup(Guid ID);

        #endregion

        #region Endpoint Groups

        [OperationContract]
        EndpointGroup CreateEndpointGroup(EndpointGroup endpointGroup);

        [OperationContract]
        EndpointGroup UpdateEndpointGroup(EndpointGroup operationGroup);

        [OperationContract]
        EndpointGroup GetEndpointGroup(Guid ID);

        [OperationContract]
        void DeleteEndpointGroup(Guid ID);

        [OperationContract]
        EntityGroupList ListEndpointGroups();

        [OperationContract]
        EntityGroupList ListEndpointGroupsByEndpoint(Guid endpointID);

        [OperationContract]
        List<EndpointListElement> ListEndpointsByGroup(Guid ID);

        #endregion 

        #region MSESystem Groups

        [OperationContract]
        MSESystemGroup CreateMSESystemGroup(MSESystemGroup mseSystemGroup);

        [OperationContract]
        MSESystemGroup UpdateMSESystemGroup(MSESystemGroup mseSystemGroup);

        [OperationContract]
        MSESystemGroup GetMSESystemGroup(Guid ID);

        [OperationContract]
        void DeleteMSESystemGroup(Guid ID);

        [OperationContract]
        EntityGroupList ListMSESystemGroups();

        [OperationContract]
        EntityGroupList ListMSESystemGroupsByMSESystem(Guid mseSystemID);

        [OperationContract]
        List<MSESystemListElement> ListMSESystemsByGroup(Guid ID);

        #endregion

        #region SystemInstance Groups

        [OperationContract]
        SystemInstanceGroup CreateSystemInstanceGroup(SystemInstanceGroup systemInstanceGroup);

        [OperationContract]
        SystemInstanceGroup UpdateSystemInstanceGroup(SystemInstanceGroup systemInstanceGroup);

        [OperationContract]
        SystemInstanceGroup GetSystemInstanceGroup(Guid ID);

        [OperationContract]
        void DeleteSystemInstanceGroup(Guid ID);

        [OperationContract]
        EntityGroupList ListSystemInstanceGroups();

        [OperationContract]
        EntityGroupList ListSystemInstanceGroupsBySystemInstance(Guid systemInstanceID);

        [OperationContract]
        List<SystemInstanceListElement> ListSystemInstancesByGroup(Guid ID);

        #endregion

        #region Resource Groups

        [OperationContract]
        ResourceGroup CreateResourceGroup(ResourceGroup resourceGroup);

        [OperationContract]
        ResourceGroup UpdateResourceGroup(ResourceGroup resourceGroup);

        [OperationContract]
        ResourceGroup GetResourceGroup(Guid ID);

        [OperationContract]
        void DeleteResourceGroup(Guid ID);

        [OperationContract]
        EntityGroupList ListResourceGroups();

        [OperationContract]
        EntityGroupList ListResourceGroupsByResource(Guid resourceID);

        [OperationContract]
        List<ResourceListElement> ListResourcesByGroup(Guid ID);

        #endregion 

        #region MSEBinding Groups

        [OperationContract]
        MSEBindingGroup CreateMSEBindingGroup(MSEBindingGroup mseBinding);

        [OperationContract]
        MSEBindingGroup UpdateMSEBindingGroup(MSEBindingGroup mseBinding);

        [OperationContract]
        MSEBindingGroup GetMSEBindingGroup(Guid ID);

        [OperationContract]
        void DeleteMSEBindingGroup(Guid ID);

        [OperationContract]
        EntityGroupList ListMSEBindingGroups();

        [OperationContract]
        EntityGroupList ListMSEBindingGroupsByMSEBinding(Guid mseBindingID);

        [OperationContract]
        List<MSEBindingListElement> ListMSEBindingsByGroup(Guid ID);

        #endregion

        #region Schema Groups

        [OperationContract]
        SchemaGroup CreateSchemaGroup(SchemaGroup schemaGroup);

        [OperationContract]
        SchemaGroup UpdateSchemaGroup(SchemaGroup schemaGroup);

        [OperationContract]
        SchemaGroup GetSchemaGroup(Guid ID);

        [OperationContract]
        void DeleteSchemaGroup(Guid ID);

        [OperationContract]
        EntityGroupList ListSchemaGroups();

        [OperationContract]
        EntityGroupList ListSchemaGroupsBySchema(Guid schemaID);

        [OperationContract]
        List<SchemaListElement> ListSchemasByGroup(Guid ID);

        #endregion

        #region DataEntity Groups

        [OperationContract]
        DataEntityGroup CreateDataEntityGroup(DataEntityGroup dataEntityGroup);

        [OperationContract]
        DataEntityGroup UpdateDataEntityGroup(DataEntityGroup dataEntityGroup);

        [OperationContract]
        DataEntityGroup GetDataEntityGroup(Guid ID);

        [OperationContract]
        void DeleteDataEntityGroup(Guid ID);

        [OperationContract]
        EntityGroupList ListDataEntityGroups();

        [OperationContract]
        EntityGroupList ListDataEntityGroupsByDataEntity(Guid dataEntityID);

        [OperationContract]
        List<DataEntityListElement> ListDataEntitiesByGroup(Guid ID);

        #endregion

        #region Runtime Server Groups

        [OperationContract]
        RuntimeServerGroup CreateRuntimeServerGroup(RuntimeServerGroup runtimeServerGroup);

        [OperationContract]
        RuntimeServerGroup UpdateRuntimeServerGroup(RuntimeServerGroup runtimeServerGroup);

        [OperationContract]
        RuntimeServerGroup GetRuntimeServerGroup(Guid ID);

        [OperationContract]
        void DeleteRuntimeServerGroup(Guid ID);

        [OperationContract]
        EntityGroupList ListRuntimeServerGroups();

        [OperationContract]
        EntityGroupList ListRuntimeServerGroupsByRuntimeServer(Guid runtimeServerID);

        [OperationContract]
        List<RuntimeServerListElement> ListRuntimeServersByGroup(Guid ID);

        #endregion       

        #region Policy Groups

        [OperationContract]
        PolicyGroup CreatePolicyGroup(PolicyGroup poicyGroup);

        [OperationContract]
        PolicyGroup UpdatePolicyGroup(PolicyGroup policyGroup);

        [OperationContract]
        PolicyGroup GetPolicyGroup(Guid ID);

        [OperationContract]
        void DeletePolicyGroup(Guid ID);

        [OperationContract]
        EntityGroupList ListPolicyGroups();

        [OperationContract]
        EntityGroupList ListPolicyGroupsByPolicy(Guid policyID);

        [OperationContract]
        List<PolicyListElement> ListPolicysByGroup(Guid ID);

        #endregion

        #region Policy Assertion Groups

        [OperationContract]
        PolicyAssertionGroup CreatePolicyAssertionGroup(PolicyAssertionGroup policyAssertionGroup);

        [OperationContract]
        PolicyAssertionGroup UpdatePolicyAssertionGroup(PolicyAssertionGroup policyAssertionGroup);

        [OperationContract]
        PolicyAssertionGroup GetPolicyAssertionGroup(Guid ID);

        [OperationContract]
        void DeletePolicyAssertionGroup(Guid ID);

        [OperationContract]
        EntityGroupList ListPolicyAssertionGroups();

        [OperationContract]
        EntityGroupList ListPolicyAssertionGroupsByPolicyAssertion(Guid policyAssertionID);

        [OperationContract]
        List<PolicyAssertionListElement> ListPolicyAssertionsByGroup(Guid ID);

        #endregion

        #region Assertion Type Groups

        [OperationContract]
        AssertionTypeGroup CreateAssertionTypeGroup(AssertionTypeGroup assertionTypeGroup);

        [OperationContract]
        AssertionTypeGroup UpdateAssertionTypeGroup(AssertionTypeGroup assertionTypeGroup);

        [OperationContract]
        AssertionTypeGroup GetAssertionTypeGroup(Guid ID);

        [OperationContract]
        void DeleteAssertionTypeGroup(Guid ID);

        [OperationContract]
        EntityGroupList ListAssertionTypeGroups();

        [OperationContract]
        EntityGroupList ListAssertionTypeGroupsByAssertionType(Guid assertionTypeID);

        [OperationContract]
        List<AssertionTypeListElement> ListAssertionTypesByGroup(Guid ID);

        #endregion

        //Utility Methods

        #region Archiver Support Methods

        [OperationContract]
        SchemaObject GetEquivalentSchemaObject(SchemaObject schemaObject);

        [OperationContract]
        List<Schema> ListSchemasForExport(List<Entity> entities);

        [OperationContract]
        void SaveSchemasFromImport(List<Schema> schemas);

        [OperationContract]
        SchemaObject GetSchemaObjectByNameTypeAndNamespace(string namespaceUri, string name, SchemaObjectType type);

        [OperationContract]
        List<SchemaObject> ListSchemaObjectByNameTypeAndNamespace(string namespaceUri, string name, SchemaObjectType type);

        #endregion Archiver Support Methods

        #region UDDI Methods

        [OperationContract]
        void PublishRuntimeServerEndPointsToUddi(Guid ID, bool refresh);

        [OperationContract]
        void RemoveRuntimeServerEndPointsFromUddi(Guid ID);

        [OperationContract]
        bool IsUddiIntegrationEnabled();

        #endregion
   
    }
}
