﻿CREATE PROCEDURE [ServiceEngine].[AssignPublishedOperationVersionToEndpoint]
	@OperationVersionID uniqueidentifier,
	@EndpointID uniqueidentifier
AS
BEGIN
	DECLARE @ErrorMessage NVARCHAR(4000);
	DECLARE @ErrorSeverity INT;
	DECLARE @ErrorState INT;
	
	SET NOCOUNT ON;

	BEGIN TRY
		BEGIN TRANSACTION

		DECLARE @OperationID uniqueidentifier

		SET @OperationID = (SELECT OperationID FROM ServiceEngine.OperationVersions 
		WHERE OperationVersionID = @OperationVersionID)

		DECLARE @ID as uniqueidentifier

		SET @ID = newid();

		INSERT	ServiceEngine.Endpoints_PublishedOperations (Endpoints_PublishedOperationsID, OperationVersionID, EndpointID, OperationID)
		VALUES	(@ID, @OperationVersionID, @EndpointID, @OperationID)

		COMMIT TRANSACTION
	END TRY
	BEGIN CATCH
		SELECT 
			@ErrorMessage = ERROR_MESSAGE(),
			@ErrorSeverity = ERROR_SEVERITY(),
			@ErrorState = ERROR_STATE();

		IF XACT_STATE() <> 0 ROLLBACK TRANSACTION;

		RAISERROR (@ErrorMessage, 
				   @ErrorSeverity,
				   1);
	END CATCH
END


