﻿CREATE PROCEDURE [ServiceEngine].[CreateMessagePart]
	@OperationVersionID uniqueidentifier,
	@IsRequestMsg bit,
	@Ordinal int,
	@SchemaObjectID uniqueidentifier,
	@ID uniqueidentifier OUTPUT
AS
BEGIN
	DECLARE @ErrorMessage NVARCHAR(4000);
	DECLARE @ErrorSeverity INT;
	DECLARE @ErrorState INT;
	SET NOCOUNT ON;

	SET @ID = newid();

	BEGIN TRY
		BEGIN TRANSACTION

		INSERT INTO ServiceEngine.MessageParts (MessagePartID, OperationVersionID, IsRequestMsg, Ordinal, SchemaObjectID)
		VALUES (@ID, @OperationVersionID, @IsRequestMsg, @Ordinal, @SchemaObjectID)

		COMMIT TRANSACTION
	END TRY
	BEGIN CATCH
		SELECT 
			@ErrorMessage = ERROR_MESSAGE(),
			@ErrorSeverity = ERROR_SEVERITY(),
			@ErrorState = ERROR_STATE();

		IF XACT_STATE() <> 0 ROLLBACK TRANSACTION;

		RAISERROR (@ErrorMessage, 
				   @ErrorSeverity,
				   1);
	END CATCH
END


