﻿CREATE PROCEDURE [ServiceEngine].[CreateOperation]
	@Name nvarchar(400),
	@PolicyID uniqueidentifier = null,
	@Description ntext = null,
	@ID uniqueidentifier OUTPUT
AS
BEGIN
	DECLARE @ErrorMessage NVARCHAR(4000);
	DECLARE @ErrorSeverity INT;
	DECLARE @ErrorState INT;
	SET NOCOUNT ON;

	BEGIN TRY
		BEGIN TRANSACTION

		SET @ID = newid();

		BEGIN TRY
			INSERT INTO ServiceEngine.Names (EntityID, EntityTypeID, EntityName, EntityDescription)
			VALUES (@ID, 4, @Name, @Description)
		END TRY
		BEGIN CATCH

			SELECT 
				@ErrorMessage = 'Attempt to insert Operation with duplicate name failed',
				@ErrorSeverity = 11,
				@ErrorState = ERROR_STATE();

			RAISERROR (@ErrorMessage, 
					   @ErrorSeverity,
					   1);
		END CATCH

		INSERT INTO ServiceEngine.Operations (OperationID, PolicyID)
		VALUES (@ID, @PolicyID)

		COMMIT TRANSACTION
	END TRY
	BEGIN CATCH
		SELECT 
			@ErrorMessage = ERROR_MESSAGE(),
			@ErrorSeverity = ERROR_SEVERITY(),
			@ErrorState = ERROR_STATE();

		IF XACT_STATE() <> 0 ROLLBACK TRANSACTION;

		RAISERROR (@ErrorMessage, 
				   @ErrorSeverity,
				   1);
	END CATCH
END