﻿CREATE PROCEDURE [ServiceEngine].[CreateSystem]
	@Name nvarchar(400),	
	@OwnerName nvarchar(200) = null,
	@Description ntext = null,		
	@OwnerContact nvarchar(200) = null,
	@Location nvarchar(max) = null,
	@PolicyID uniqueidentifier = null,
	@URI NVARCHAR (500)=NULL,
	@ID uniqueidentifier OUTPUT
AS
BEGIN
	DECLARE @ErrorMessage NVARCHAR(4000);
	DECLARE @ErrorSeverity INT;
	DECLARE @ErrorState INT;
	SET NOCOUNT ON;

	IF @ID IS NULL 
		SET @ID = newid();

	BEGIN TRY
		BEGIN TRANSACTION

		INSERT INTO ServiceEngine.Systems (SystemID, OwnerName, OwnerContact, Location, PolicyID, URI)
		VALUES (@ID, @OwnerName, @OwnerContact, @Location, @PolicyID, @URI)

		BEGIN TRY
			INSERT INTO ServiceEngine.Names (EntityID, EntityTypeID, EntityName, EntityDescription)
			VALUES (@ID, 12, @Name, @Description)
		END TRY
		BEGIN CATCH

			SELECT 
				@ErrorMessage = 'Attempt to insert System with duplicate name failed',
				@ErrorSeverity = 11,
				@ErrorState = ERROR_STATE();

			RAISERROR (@ErrorMessage, 
					   @ErrorSeverity,
					   1);
		END CATCH

		COMMIT TRANSACTION
	END TRY
	BEGIN CATCH
		SELECT 
			@ErrorMessage = ERROR_MESSAGE(),
			@ErrorSeverity = ERROR_SEVERITY(),
			@ErrorState = ERROR_STATE();

		IF XACT_STATE() <> 0 ROLLBACK TRANSACTION;

		RAISERROR (@ErrorMessage, 
				   @ErrorSeverity,
				   1);
	END CATCH

END