﻿CREATE PROCEDURE [ServiceEngine].[CreateSystemInstance]
	@SystemID uniqueidentifier = NULL,
	@Name nvarchar(400),	
	@OwnerName nvarchar(200) = NULL,
	@Description ntext = NULL,		
	@OwnerContact nvarchar(200) = NULL,
	@URI nvarchar(500) = NULL,
	@PolicyID uniqueidentifier = NULL,
	@Port int = NULL,
	@ID uniqueidentifier OUTPUT
AS
BEGIN
	DECLARE @ErrorMessage NVARCHAR(4000);
	DECLARE @ErrorSeverity INT;
	DECLARE @ErrorState INT;
	SET NOCOUNT ON;

	IF @ID IS NULL 
		SET @ID = newid();

	BEGIN TRY
		BEGIN TRANSACTION

		INSERT INTO ServiceEngine.SystemInstances (InstanceID, SystemID, OwnerName, OwnerContact, URI, PolicyID, Port)
		VALUES (@ID, @SystemID, @OwnerName, @OwnerContact, @URI, @PolicyID, @Port)

		BEGIN TRY
			INSERT INTO ServiceEngine.Names (EntityID, EntityTypeID, EntityName, EntityDescription)
			VALUES (@ID, 14, @Name, @Description)
		END TRY
		BEGIN CATCH

			SELECT 
				@ErrorMessage = 'Attempt to insert System with duplicate name failed',
				@ErrorSeverity = 11,
				@ErrorState = ERROR_STATE();

			RAISERROR (@ErrorMessage, 
					   @ErrorSeverity,
					   1);
		END CATCH

		COMMIT TRANSACTION
	END TRY
	BEGIN CATCH
		SELECT 
			@ErrorMessage = ERROR_MESSAGE(),
			@ErrorSeverity = ERROR_SEVERITY(),
			@ErrorState = ERROR_STATE();

		IF XACT_STATE() <> 0 ROLLBACK TRANSACTION;

		RAISERROR (@ErrorMessage, 
				   @ErrorSeverity,
				   1);
	END CATCH

END