﻿CREATE PROCEDURE [ServiceEngine].[UpdateChannel]
	@Name nvarchar(400),
	@MonikerSchema xml,
	@BindingID uniqueidentifier,
	@EditorAssembly nvarchar(max),
	@EditorTypeName nvarchar(max),
	@Description ntext = null,
	@ID uniqueidentifier 
AS
BEGIN
	DECLARE @ErrorMessage NVARCHAR(4000);
	DECLARE @ErrorSeverity INT;
	DECLARE @ErrorState INT;
	SET NOCOUNT ON;

	BEGIN TRY
		BEGIN TRANSACTION

		UPDATE	ServiceEngine.Channels 
		SET		MonikerSchema = @MonikerSchema,
				BindingID = @BindingID,
				EditorAssembly = @EditorAssembly,
				EditorTypeName = @EditorTypeName
		WHERE	ChannelID = @ID

		UPDATE	ServiceEngine.Names
		SET		EntityName = @Name,
				EntityDescription = @Description 
		WHERE	EntityID = @ID and EntityTypeID = 7

		COMMIT TRANSACTION
	END TRY
	BEGIN CATCH
		SELECT 
			@ErrorMessage = ERROR_MESSAGE(),
			@ErrorSeverity = ERROR_SEVERITY(),
			@ErrorState = ERROR_STATE();

		IF XACT_STATE() <> 0 ROLLBACK TRANSACTION;

		RAISERROR (@ErrorMessage, 
				   @ErrorSeverity,
				   1);
	END CATCH
END