﻿CREATE PROCEDURE [ServiceEngine].[UpdateSchemaObject]
@schemaObjectID UNIQUEIDENTIFIER, @schemaID UNIQUEIDENTIFIER, @schemaObjectType SMALLINT, @schemaObjectName NVARCHAR (256), @schemaObjectXml XML, @parentID UNIQUEIDENTIFIER=NULL, @ID UNIQUEIDENTIFIER OUTPUT
AS
BEGIN
	DECLARE @ErrorMessage NVARCHAR(4000);
	DECLARE @ErrorSeverity INT;
	DECLARE @ErrorState INT;
	DECLARE @entityID UNIQUEIDENTIFIER;
	
	SET NOCOUNT ON;

	BEGIN TRY
		BEGIN
			BEGIN TRANSACTION
				--SET the Revision Date on existing SchemaObject
				SELECT @entityID = DataEntityID
				FROM [SchemaObjects]
				WHERE [SchemaObjectID] = @schemaObjectID

				UPDATE [SchemaObjects]
				SET DateRevised = GetDate()
				WHERE [SchemaObjectID] = @schemaObjectID
				--Create the new SchemaObject
				SET @ID = newid();
				INSERT INTO [SchemaObjects]
						   ([SchemaObjectID]
						   ,[SchemaID]
						   ,[SchemaObjectType]
						   ,[SchemaObjectName]
						   ,[SchemaObjectXml]
						   ,DataEntityID)
					 VALUES
						   (@ID
						   ,@schemaID
						   ,@schemaObjectType
						   ,@schemaObjectName
						   ,@schemaObjectXml
						   ,@entityID)
						   
				IF (@parentID IS NOT NULL)
				BEGIN
					INSERT INTO [SchemaObjectDependencies]
						   ([SchemaObjectDependencyID]
						   ,[SchemaObjectID]
						   ,[ParentObjectID])
					 VALUES
						   (newID()
						   ,@ID
						   ,@parentID)
				END
			COMMIT TRANSACTION
		END
	END TRY
	BEGIN CATCH
		SELECT 
			@ErrorMessage = ERROR_MESSAGE(),
			@ErrorSeverity = ERROR_SEVERITY(),
			@ErrorState = ERROR_STATE();

		IF XACT_STATE() <> 0 ROLLBACK TRANSACTION;

		RAISERROR (@ErrorMessage, 
				   @ErrorSeverity,
				   1);
	END CATCH
END

