﻿CREATE PROCEDURE [ServiceEngine].[UpdateSchemaObjectReferences]
@OldSchemaObjectID UNIQUEIDENTIFIER, @newSchemaObjectID UNIQUEIDENTIFIER
AS
BEGIN
	DECLARE @ErrorMessage NVARCHAR(4000);
	DECLARE @ErrorSeverity INT;
	DECLARE @ErrorState INT;
	DECLARE @entityID UNIQUEIDENTIFIER;
	
	SET NOCOUNT ON;

	BEGIN TRY
		BEGIN
			BEGIN TRANSACTION
				UPDATE 
					[SchemaObjectDependencies]
				SET 
					[SchemaObjectID] = @newSchemaObjectID
				WHERE 
					[SchemaObjectID] = @OldSchemaObjectID
			COMMIT TRANSACTION
		END
	END TRY
	BEGIN CATCH
		SELECT 
			@ErrorMessage = ERROR_MESSAGE(),
			@ErrorSeverity = ERROR_SEVERITY(),
			@ErrorState = ERROR_STATE();

		IF XACT_STATE() <> 0 ROLLBACK TRANSACTION;

		RAISERROR (@ErrorMessage, 
				   @ErrorSeverity,
				   1);
	END CATCH
END

