﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ServiceModel;
using System.ServiceModel.Channels;
using Microsoft.MSE.Catalog.Contracts;
using System.ServiceModel.Description;

namespace Microsoft.MSE.Catalog.Module
{
    public class Notification
    {
        private string mURL;
        private string mReferenceParameters;
        private string mAction;
        private Binding mBinding;
        private MseEventData mEventData;
        public Notification(MseEventData eventData, string url,
                                    string action, string referenceParameters,
                                    Binding binding)
        {
            mEventData = eventData;
            mURL = url;
            mReferenceParameters = referenceParameters;
            mAction = action;
            mBinding = binding;
        }

        private AcceptEventRequest BuildNotificationMessage()
        {
            AcceptEventRequest msg = new AcceptEventRequest(mEventData);
            return msg;
        }

        private TChannel BuildChannel<TChannel>(Binding binding, EndpointAddress endpointAddress) where TChannel : IChannel
        {
            ChannelFactory<TChannel> fac = new ChannelFactory<TChannel>(binding);
            // NZ TODO
            // We need to determine based upon the Binding if we should allow session or not

            try
            {
                fac.Endpoint.Contract.SessionMode = SessionMode.Allowed;
            }
            catch
            { }

            TChannel channel = fac.CreateChannel(endpointAddress);

            return channel;
        }

        public void SendOneWayNotification()
        {
            try
            {
                TraceHelper.TraceInformation("SendOneWayNotification Executed for URL [{0}]", mURL);

                AcceptEventRequest eventMessage = BuildNotificationMessage();
                IOutputChannel channel = BuildChannel<IOutputChannel>(mBinding, new EndpointAddress(mURL));

                TypedMessageConverter tmc = TypedMessageConverter.Create(typeof(AcceptEventRequest), mAction, SC.MSENamespace);
                Message msg = tmc.ToMessage(eventMessage, mBinding.MessageVersion);

                try
                {
                    channel.Send(msg);
                }
                finally
                {
                    try
                    {
                        channel.Close();
                    }
                    catch { }
                }
            }
            catch (CommunicationException commEx)
            {
                TraceHelper.TraceError("SendOneWayNotification for URL [{0}] and Action [{1}] failed with communication error [{2}]", mURL, mAction, CatalogSvcHelper.GetExceptionDetails(commEx));
            }
            catch (Exception ex)
            {
                TraceHelper.TraceError("SendOneWayNotification for URL [{0}] and Action [{1}] failed with general error [{2}]", mURL, mAction, CatalogSvcHelper.GetExceptionDetails(ex));
            }
        }
        public void SendNotification()
        {
            try
            {
                TraceHelper.TraceInformation("SendNotification Executed for URL [{0}]", mURL);

                AcceptEventRequest eventMessage = BuildNotificationMessage();

                IRequestChannel channel = BuildChannel<IRequestChannel>(mBinding, new EndpointAddress(mURL));

                TypedMessageConverter tmc = TypedMessageConverter.Create(typeof(AcceptEventRequest), mAction, SC.MSENamespace);
                Message msg = tmc.ToMessage(eventMessage, mBinding.MessageVersion);

                try
                {
                    channel.Request(msg);
                }
                finally
                {
                    try
                    {
                        channel.Close();
                    }
                    catch { }
                }
            }
            catch (CommunicationException commEx)
            {
                TraceHelper.TraceError("SendNotification for URL [{0}] and Action [{1}] failed with communication error [{2}]", mURL, mAction, CatalogSvcHelper.GetExceptionDetails(commEx));
            }
            catch (Exception ex)
            {
                TraceHelper.TraceError("SendNotification for URL [{0}] and Action [{1}] failed with general error [{2}]", mURL, mAction, CatalogSvcHelper.GetExceptionDetails(ex));
            }
        }
    }
}
